@@ -1,12 +1,22 @@
 package List::AllUtils;
+{
+  $List::AllUtils::VERSION = '0.04';
+}
+BEGIN {
+  $List::AllUtils::AUTHORITY = 'cpan:DROLSKY';
+}
 
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
-
-use List::Util qw( first min max minstr maxstr reduce sum shuffle );
-use List::MoreUtils qw( :all );
+# List::Util does not define an :all tag
+BEGIN
+{
+    require List::Util;
+    List::Util->VERSION(1.1);
+    List::Util->import( @List::Util::EXPORT_OK );
+}
+use List::MoreUtils 0.28 qw( :all );
 
 use base 'Exporter';
 
@@ -17,6 +27,8 @@ our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
 1;
 
+# ABSTRACT: Combines List::Util and List::MoreUtils in one bite-sized package
+
 __END__
 
 =pod
@@ -25,6 +37,10 @@ __END__
 
 List::AllUtils - Combines List::Util and List::MoreUtils in one bite-sized package
 
+=head1 VERSION
+
+version 0.04
+
 =head1 SYNOPSIS
 
     use List::AllUtils qw( first any );
@@ -42,9 +58,38 @@ defined in C<List::Util> or C<List::MoreUtils>? I sure am. Now you
 don't have to remember. This module will export all of the functions
 that either of those two modules defines.
 
-=head1 FUNCTIONS
+Note that all function documentation has been shamelessly copied from
+L<List::Util> and <List::MoreUtils>.
 
-(Shamelessly copied from List::Util and List::MoreUtils ...)
+=head1 LIST-REDUCTION FUNCTIONS
+
+The following set of functions all reduce a list down to a single value.
+
+=head2 reduce BLOCK LIST
+
+Reduces LIST by calling BLOCK, in a scalar context, multiple times,
+setting C<$a> and C<$b> each time. The first call will be with C<$a>
+and C<$b> set to the first two elements of the list, subsequent
+calls will be done by setting C<$a> to the result of the previous
+call and C<$b> to the next element in the list.
+
+Returns the result of the last call to BLOCK. If LIST is empty then
+C<undef> is returned. If LIST only contains one element then that
+element is returned and BLOCK is not executed.
+
+    $foo = reduce { $a < $b ? $a : $b } 1..10       # min
+    $foo = reduce { $a lt $b ? $a : $b } 'aa'..'zz' # minstr
+    $foo = reduce { $a + $b } 1 .. 10               # sum
+    $foo = reduce { $a . $b } @bar                  # concat
+
+If your algorithm requires that C<reduce> produce an identity value, then
+make sure that you always pass that identity value as the first argument to prevent
+C<undef> being returned
+
+  $foo = reduce { $a + $b } 0, @values;             # sum with 0 identity value
+
+The remaining list-reduction functions are all specialisations of this
+generic idea.
 
 =head2 first BLOCK LIST
 
@@ -59,7 +104,7 @@ C<undef> is returned.
 
 This function could be implemented using C<reduce> like this
 
-    $foo = reduce { wanted($a) ? $a : wanted($b) ? $b : undef } undef, @list
+    $foo = reduce { defined($a) ? $a : wanted($b) ? $b : undef } undef, @list
 
 for example wanted() could be defined() which would return the first
 defined value in @list
@@ -118,29 +163,6 @@ This function could be implemented using C<reduce> like this
 
     $foo = reduce { $a lt $b ? $a : $b } 'A'..'Z'
 
-=head2 reduce BLOCK LIST
-
-Reduces LIST by calling BLOCK, in a scalar context, multiple times,
-setting C<$a> and C<$b> each time. The first call will be with C<$a>
-and C<$b> set to the first two elements of the list, subsequent
-calls will be done by setting C<$a> to the result of the previous
-call and C<$b> to the next element in the list.
-
-Returns the result of the last call to BLOCK. If LIST is empty then
-C<undef> is returned. If LIST only contains one element then that
-element is returned and BLOCK is not executed.
-
-    $foo = reduce { $a < $b ? $a : $b } 1..10       # min
-    $foo = reduce { $a lt $b ? $a : $b } 'aa'..'zz' # minstr
-    $foo = reduce { $a + $b } 1 .. 10               # sum
-    $foo = reduce { $a . $b } @bar                  # concat
-
-=head2 shuffle LIST
-
-Returns the elements of LIST in a random order
-
-    @cards = shuffle 0..51      # 0..51 in a random order
-
 =head2 sum LIST
 
 Returns the sum of all the elements in LIST. If LIST is empty then
@@ -154,57 +176,167 @@ This function could be implemented using C<reduce> like this
 
     $foo = reduce { $a + $b } 1..10
 
+=head2 sum0 LIST
+
+Similar to C<sum>, except this returns 0 when given an empty list, rather
+than C<undef>.
+
+=head1 KEY/VALUE PAIR LIST FUNCTIONS
+
+The following set of functions, all inspired by L<List::Pairwise>, consume
+an even-sized list of pairs. The pairs may be key/value associations from a
+hash, or just a list of values. The functions will all preserve the original
+ordering of the pairs, and will not be confused by multiple pairs having the
+same "key" value - nor even do they require that the first of each pair be a
+plain string.
+
+=head2 pairgrep BLOCK KVLIST
+
+Similar to perl's C<grep> keyword, but interprets the given list as an
+even-sized list of pairs. It invokes the BLOCK multiple times, in scalar
+context, with C<$a> and C<$b> set to successive pairs of values from the
+KVLIST.
+
+Returns an even-sized list of those pairs for which the BLOCK returned true
+in list context, or the count of the B<number of pairs> in scalar context.
+(Note, therefore, in scalar context that it returns a number half the size
+of the count of items it would have returned in list context).
+
+    @subset = pairgrep { $a =~ m/^[[:upper:]]+$/ } @kvlist
+
+Similar to C<grep>, C<pairgrep> aliases C<$a> and C<$b> to elements of the
+given list. Any modifications of it by the code block will be visible to
+the caller.
+
+=head2 pairfirst BLOCK KVLIST
+
+Similar to the C<first> function, but interprets the given list as an
+even-sized list of pairs. It invokes the BLOCK multiple times, in scalar
+context, with C<$a> and C<$b> set to successive pairs of values from the
+KVLIST.
+
+Returns the first pair of values from the list for which the BLOCK returned
+true in list context, or an empty list of no such pair was found. In scalar
+context it returns a simple boolean value, rather than either the key or the
+value found.
+
+    ( $key, $value ) = pairfirst { $a =~ m/^[[:upper:]]+$/ } @kvlist
+
+Similar to C<grep>, C<pairfirst> aliases C<$a> and C<$b> to elements of the
+given list. Any modifications of it by the code block will be visible to
+the caller.
+
+=head2 pairmap BLOCK KVLIST
+
+Similar to perl's C<map> keyword, but interprets the given list as an
+even-sized list of pairs. It invokes the BLOCK multiple times, in list
+context, with C<$a> and C<$b> set to successive pairs of values from the
+KVLIST.
+
+Returns the concatenation of all the values returned by the BLOCK in list
+context, or the count of the number of items that would have been returned
+in scalar context.
+
+    @result = pairmap { "The key $a has value $b" } @kvlist
+
+Similar to C<map>, C<pairmap> aliases C<$a> and C<$b> to elements of the
+given list. Any modifications of it by the code block will be visible to
+the caller.
+
+=head2 pairs KVLIST
+
+A convenient shortcut to operating on even-sized lists of pairs, this
+function returns a list of ARRAY references, each containing two items from
+the given list. It is a more efficient version of
+
+    pairmap { [ $a, $b ] } KVLIST
+
+It is most convenient to use in a C<foreach> loop, for example:
+
+    foreach ( pairs @KVLIST ) {
+       my ( $key, $value ) = @$_;
+       ...
+    }
+
+=head2 pairkeys KVLIST
+
+A convenient shortcut to operating on even-sized lists of pairs, this
+function returns a list of the the first values of each of the pairs in
+the given list. It is a more efficient version of
+
+    pairmap { $a } KVLIST
+
+=head2 pairvalues KVLIST
+
+A convenient shortcut to operating on even-sized lists of pairs, this
+function returns a list of the the second values of each of the pairs in
+the given list. It is a more efficient version of
+
+    pairmap { $b } KVLIST
+
+=head1 OTHER FUNCTIONS
+
+=head2 shuffle LIST
+
+Returns the elements of LIST in a random order
+
+    @cards = shuffle 0..51      # 0..51 in a random order
+
+=head1 List::MoreUtils FUNCTIONS
+
 =head2 any BLOCK LIST
 
 Returns a true value if any item in LIST meets the criterion given through
 BLOCK. Sets C<$_> for each item in LIST in turn:
 
     print "At least one value undefined"
-        if any { !defined($_) } @list;
+        if any { ! defined($_) } @list;
 
-Returns false otherwise, or C<undef> if LIST is empty.
+Returns false otherwise, or if LIST is empty.
 
 =head2 all BLOCK LIST
 
 Returns a true value if all items in LIST meet the criterion given through
-BLOCK. Sets C<$_> for each item in LIST in turn:
+BLOCK, or if LIST is empty. Sets C<$_> for each item in LIST in turn:
 
     print "All items defined"
         if all { defined($_) } @list;
 
-Returns false otherwise, or C<undef> if LIST is empty.
+Returns false otherwise.
 
 =head2 none BLOCK LIST
 
-Logically the negation of C<any>. Returns a true value if no item in LIST meets the
-criterion given through BLOCK. Sets C<$_> for each item in LIST in turn:
+Logically the negation of C<any>. Returns a true value if no item in LIST meets
+the criterion given through BLOCK, or if LIST is empty. Sets C<$_> for each item
+in LIST in turn:
 
     print "No value defined"
         if none { defined($_) } @list;
 
-Returns false otherwise, or C<undef> if LIST is empty.
+Returns false otherwise.
 
 =head2 notall BLOCK LIST
 
-Logically the negation of C<all>. Returns a true value if not all items in LIST meet
-the criterion given through BLOCK. Sets C<$_> for each item in LIST in turn:
+Logically the negation of C<all>. Returns a true value if not all items in LIST
+meet the criterion given through BLOCK. Sets C<$_> for each item in LIST in
+turn:
 
     print "Not all values defined"
         if notall { defined($_) } @list;
 
-Returns false otherwise, or C<undef> if LIST is empty.
+Returns false otherwise, or if LIST is empty.
 
 =head2 true BLOCK LIST
 
-Counts the number of elements in LIST for which the criterion in BLOCK is true. Sets C<$_> for 
-each item in LIST in turn:
+Counts the number of elements in LIST for which the criterion in BLOCK is true.
+Sets C<$_> for  each item in LIST in turn:
 
     printf "%i item(s) are defined", true { defined($_) } @list;
 
 =head2 false BLOCK LIST
 
-Counts the number of elements in LIST for which the criterion in BLOCK is false. Sets C<$_> for
-each item in LIST in turn:
+Counts the number of elements in LIST for which the criterion in BLOCK is false.
+Sets C<$_> for each item in LIST in turn:
 
     printf "%i item(s) are not defined", false { defined($_) } @list;
 
@@ -212,8 +344,8 @@ each item in LIST in turn:
 
 =head2 first_index BLOCK LIST
 
-Returns the index of the first element in LIST for which the criterion in BLOCK is true. Sets C<$_>
-for each item in LIST in turn:
+Returns the index of the first element in LIST for which the criterion in BLOCK
+is true. Sets C<$_> for each item in LIST in turn:
 
     my @list = (1, 4, 3, 2, 4, 6);
     printf "item with index %i in list is 4", firstidx { $_ == 4 } @list;
@@ -228,8 +360,8 @@ C<first_index> is an alias for C<firstidx>.
 
 =head2 last_index BLOCK LIST
 
-Returns the index of the last element in LIST for which the criterion in BLOCK is true. Sets C<$_>
-for each item in LIST in turn:
+Returns the index of the last element in LIST for which the criterion in BLOCK
+is true. Sets C<$_> for each item in LIST in turn:
 
     my @list = (1, 4, 3, 2, 4, 6);
     printf "item with index %i in list is 4", lastidx { $_ == 4 } @list;
@@ -242,8 +374,8 @@ C<last_index> is an alias for C<lastidx>.
 
 =head2 insert_after BLOCK VALUE LIST
 
-Inserts VALUE after the first item in LIST for which the criterion in BLOCK is true. Sets C<$_> for
-each item in LIST in turn.
+Inserts VALUE after the first item in LIST for which the criterion in BLOCK is
+true. Sets C<$_> for each item in LIST in turn.
 
     my @list = qw/This is a list/;
     insert_after { $_ eq "a" } "longer" => @list;
@@ -253,7 +385,7 @@ each item in LIST in turn.
 
 =head2 insert_after_string STRING VALUE LIST
 
-Inserts VALUE after the first item in LIST which is equal to STRING. 
+Inserts VALUE after the first item in LIST which is equal to STRING.
 
     my @list = qw/This is a list/;
     insert_after_string "a", "longer" => @list;
@@ -280,6 +412,15 @@ Think of it as syntactic sugar for
 
     for (my @mult = @list) { $_ *= 2 }
 
+=head2 before BLOCK LIST
+
+Returns a list of values of LIST up to (and not including) the point where BLOCK
+returns a true value. Sets C<$_> for each element in LIST in turn.
+
+=head2 before_incl BLOCK LIST
+
+Same as C<before> but also includes the element for which BLOCK is true.
+
 =head2 after BLOCK LIST
 
 Returns a list of the values of LIST after (and not including) the point
@@ -289,16 +430,7 @@ where BLOCK returns a true value. Sets C<$_> for each element in LIST in turn.
 
 =head2 after_incl BLOCK LIST
 
-Same as C<after> but also inclues the element for which BLOCK is true.
-
-=head2 before BLOCK LIST
-
-Returns a list of values of LIST upto (and not including) the point where BLOCK
-returns a true value. Sets C<$_> for each element in LIST in turn.
-
-=head2 before_incl BLOCK LIST
-
-Same as C<before> but also includes the element for which BLOCK is true.
+Same as C<after> but also includes the element for which BLOCK is true.
 
 =head2 indexes BLOCK LIST
 
@@ -358,7 +490,7 @@ This is useful for looping over more than one array at once:
 
 The iterator returns the empty list when it reached the end of all arrays.
 
-If the iterator is passed an argument of 'C<index>', then it retuns
+If the iterator is passed an argument of 'C<index>', then it returns
 the index of the last fetched set of values, as a scalar.
 
 =head2 each_arrayref LIST
@@ -366,7 +498,7 @@ the index of the last fetched set of values, as a scalar.
 Like each_array, but the arguments are references to arrays, not the
 plain arrays.
 
-=head2 natatime BLOCK LIST
+=head2 natatime EXPR, LIST
 
 Creates an array iterator, for looping over an array in chunks of
 C<$n> items at a time.  (n at a time, get it?).  An example is
@@ -409,6 +541,8 @@ C<zip> is an alias for C<mesh>.
 
 =head2 uniq LIST
 
+=head2 distinct LIST
+
 Returns a new list by stripping duplicate values in LIST. The order of
 elements in the returned list is the same as in LIST. In scalar context,
 returns the number of unique elements in LIST.
@@ -419,22 +553,23 @@ returns the number of unique elements in LIST.
 =head2 minmax LIST
 
 Calculates the minimum and maximum of LIST and returns a two element list with
-the first element being the minimum and the second the maximum. Returns the empty
-list if LIST was empty.
+the first element being the minimum and the second the maximum. Returns the
+empty list if LIST was empty.
 
-The minmax algorithm differs from a naive iteration over the list where each element
-is compared to two values being the so far calculated min and max value in that it
-only requires 3n/2 - 2 comparisons. Thus it is the most efficient possible algorithm.
+The C<minmax> algorithm differs from a naive iteration over the list where each
+element is compared to two values being the so far calculated min and max value
+in that it only requires 3n/2 - 2 comparisons. Thus it is the most efficient
+possible algorithm.
 
 However, the Perl implementation of it has some overhead simply due to the fact
 that there are more lines of Perl code involved. Therefore, LIST needs to be
-fairly big in order for minmax to win over a naive implementation. This
+fairly big in order for C<minmax> to win over a naive implementation. This
 limitation does not apply to the XS version.
 
 =head2 part BLOCK LIST
 
-Partitions LIST based on the return value of BLOCK which denotes into which partition
-the current value is put.
+Partitions LIST based on the return value of BLOCK which denotes into which
+partition the current value is put.
 
 Returns a list of the partitions thusly created. Each partition created is a
 reference to an array.
@@ -455,9 +590,9 @@ Be careful with negative values, though:
 
 Negative values are only ok when they refer to a partition previously created:
 
-    my @idx = (0, 1, -1);
-    my $i = 0;
-    my @part = part { $idx[$++ % 3] } 1 .. 8;	# [1, 4, 7], [2, 3, 5, 6, 8]
+    my @idx  = ( 0, 1, -1 );
+    my $i    = 0;
+    my @part = part { $idx[$++ % 3] } 1 .. 8; # [1, 4, 7], [2, 3, 5, 6, 8]
 
 =head1 EXPORTS
 
@@ -471,13 +606,6 @@ C<List::Util> and C<List::MoreUtils>, obviously.
 Also see C<Util::Any>, which unifies many more util modules, and also
 lets you rename functions as part of the import.
 
-=head1 AUTHOR
-
-Dave Rolsky, C<< <autarch@urth.org> >>
-
-But really, this module does very little, and all the docs come from
-List::Util by Graham Barr and List::MoreUtils by Tassilo von Parseval.
-
 =head1 BUGS
 
 Please report any bugs or feature requests to
@@ -485,11 +613,16 @@ C<bug-list-allutils@rt.cpan.org>, or through the web interface at
 L<http://rt.cpan.org>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
-=head1 COPYRIGHT & LICENSE
+=head1 AUTHOR
 
-Copyright 2009 Dave Rolsky, All Rights Reserved.
+Dave Rolsky <autarch@urth.org>
 
-This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2013 by Dave Rolsky.
+
+This is free software, licensed under:
+
+  The Artistic License 2.0 (GPL Compatible)
 
 =cut
